/*******************************************************************************

	Program Name: EMC14XX.C Version 0.1
	Copyright (C) 2013 Standard Microsystems Corporation
	All rights reverved.							

	This is program for APP-ESS13-2 EVK to drive EMC1402


	Reference:																
        SMSC Data sheet EMC1402 DS Rev. 2.0 (08-10-12).pdf
===============================================================================
Revision History:

Date
Version   	who  Comment
----------  --- --------------------------------------------------------------
2013/11/16	jc	.Initial version 
V0.1			

*******************************************************************************/
#include <xc.h>
#include <stdint.h>
#include "IIC.h"
#include "GenericTypeDefs.h"



//PART			SMBUS
//NUMBER		ADDRESS
//EMC1402 - 1 1001_100xb
//EMC1402 - 2 1001_101xb
//EMC1403 - 3 0011_000xb
//EMC1404 - 4 0101_001xb
//		  - A
//#define EMCADDRESS 			// EMC14xx's slave address
								// Pull up resistor on THERM Pin
								// 4.7K ==> 0xF8
								// 6.8K ==> 0xB8
								// 10K  ==> 0x98
								// 15K  ==> 0xD8
								// 22K  ==> 0x38
								// 33K  ==> 0x78
const	uint8_t	IADD[] = { 	
		0xf8,	0xb8,	0x98,	0xd8,	0x38,	0x78,	0x9A,  0x30,   0x52 };

const	uint8_t	PID[] = { 0x20,0x21,0x25 }; 	// Product ID of EMC14X2, EMC14X3 and EMC 14X4

uint8_t	bEMCADD; 	// SMBUS Address of EMC14xx
uint8_t	bPID;	 	// number of channels of EMC14xx


#define	EMCCONFIG		0x03	// Address of Configuration register
// Data Read Interlock
// When any temperature channel high byte register is read, the corresponding low byte
// is copied into an internal ��shadow�� register. The user is free to read the low byte
// at any time and be guaranteed that it will correspond to the previously read high byte. 
// Regardless if the low byte is read or not, reading from the same high byte register again 
// will automatically refresh this stored low byte data.
#define	IDDHB			0x00	// Internal Diode Dada High byte Address
#define	IDDLB			0x29	// Internal Diode Dada Low byte Address	 B7-B5	-> 0.5, 0.25, 0.125
#define	EDDHB1			0x01	// External Diode 1 Dada High byte Address
#define	EDDLB1			0x10	// External Diode 1 Dada Low byte Address, B7-B5	-> 0.5, 0.25, 0.125
#define	EDDHB2			0x23	// External Diode 2 Dada High byte Address
#define	EDDLB2			0x24	// External Diode 2 Dada Low byte Address, B7-B5	-> 0.5, 0.25, 0.125
#define	EDDHB3			0x2a	// External Diode 3 Dada High byte Address
#define	EDDLB3			0x2b	// External Diode 3 Dada Low byte Address, B7-B5	-> 0.5, 0.25, 0.125


extern uint8_t bG5mSCnt;
const	uint8_t	TDR[] = { 	
							IDDHB,	IDDLB,		EDDHB1,	EDDLB1,
							EDDHB2,	EDDLB2,		EDDHB3,	EDDLB3
						 };
	 
//*******************************************************************************************
// ReadEMC : read temperature from EMC14xx with channel number check !
// input:  bChannel : 	0 -> Internal
//						1 -> External 1
//						2 -> External 2
//						3 -> External 3
//
// output : 11-bit (unit: 0.125 C)
// Extened Range -64 degree C to 191 degree C
//						Diode Fault             000 0000 0000
//						<= -64                  000 0000 0000
//						-1			001 0000 0000
//						0			010 0000 0000
//						0.125                   010 0000 0001
//						1			010 0000 1000
//						65			100 0000 1000
//						127			101 1111 1000
//						127.875		101 1111 1111
//						190			111 1111 0000
//						191			111 1111 1000
//						>= 191.875	111 1111 1111
//
//						0xffffffff  EMC or Channel doesn't exist
//*******************************************************************************************
int16_t ReadEMC1412( uint8_t	bChannel )
{
	WORD_VAL data;
	uint8_t	bData;

//	if ( !bPID || bChannel > bPID )
//	{
//		WriteByte( 0x50, 0x74, ReadByte(0x50, 0x74) | 0x04 ); 	// turn on LED 11 for EMC error
//		return 0xffffffff;
//	}
	data.byte.HB = ReadByte( bEMCADD, 0x01);
	data.byte.LB  = ReadByte( bEMCADD, 0x10);
        return data.Val;
}

//*************************************************************************
// Config the EMC to range -64 to 191.875 and data format is offset binary
//*************************************************************************
void InitEMC( void ) 
{
	uint8_t	i,j;

	// Search EMC's slave address
        for ( i = 0; i < sizeof ( IADD ) ; ++i){
            bEMCADD=IADD[i];
            bPID=ReadByte(bEMCADD, 0xfd);
            if(bPID==0x20){
                //WriteByte( bEMCADD, EMCCONFIG, 0b00000100 );  // set extened range
                return;
            }
        }
        
//	for ( i = 0; i < sizeof ( IADD ) ; ++i)
//	{
//		bPID=ReadByte( IADD[i], 0xfd );
//                WriteByte( bEMCADD, EMCCONFIG, 0b00000100 );  // set extened range
//		if ( bPID ) {
//			for ( j = 0; j < sizeof ( j ) ; ++j )
//			if ( bPID == PID[j] )
//			{
//				bPID = j+1; 			// total number of channels-1
//				bEMCADD = IADD[i];		// setup Smbus address
//				WriteByte( bEMCADD, EMCCONFIG, 0b00000100 );  // set extened range
//
////				WriteByte( 0x50, 0x80, 00 ); // show Address
////				WriteByte( 0x50, 0x81, 00 ); // show Address
////				WriteByte( 0x50, 0x82, 00 ); // show Address
////				WriteByte( 0x50, 0x73, bEMCADD ); // show Address
////				while(1);
//				return;
//			}
//		}
//	}
	//WriteByte( 0x50, 0x74, 0x04 ); 	// turn on LED 11 for EMC error
	//bPID = 0;
}
